--[[
LuCI support for dj
Copyright (C) 2010-2011 Thinktube Inc
]]--
local util = require "luci.util"
local nw   = require "luci.tools.network"

function gpg_list_keys()
	local gpgdir = "/etc/gnupg"
	local keys = {}
	local list = util.execi("gpg --homedir " .. gpgdir .. " --list-keys")
	local id

	if not list then
		return keys
	end

	for line in list do
		line = util.split(util.trim(line), "%s+", nil, true)
		if line[1] == "pub" and #line[2] > 0 then
			id = line[2]:match(".+/(.+)")
		elseif line[1] == "uid" and #line[2] > 0 and id then
			local key = {}
			key.id = id
			key.email = line[#line]:match("<(.+)>")
			keys[#keys+1] = key
			id = nil
		end
	end

	return keys
end

local keys = gpg_list_keys()

m = Map("dj", translate("dj_index"), translate("dj_description"))

---- app ----
app = m:section(TypedSection, "app", translate("dj_section_app", "test"))
app.anonymous = true

en = app:option(Flag, "enabled", translate("dj_enable", "Enable service"))

id = app:option(Value, "id", translate("dj_id"))
id.rmempty = false
sleep = app:option(Value, "sleepTime", translate("dj_sleepTime"), translate("seconds", "Seconds"))
sleep.rmempty = false

max_preset = app:option(Value, "maxPresetNum", translate("dj_maxPresetNum"))
max_preset.rmempty = false
preset_intvl = app:option(Value, "presetInterval", translate("dj_presetInterval"), translate("minutes", "Minutes"))
preset_intvl.rmempty = false
ss_intvl = app:option(Value, "snapshotInterval", translate("dj_snapshotInterval"), translate("seconds", "Seconds"))
ss_intvl.rmempty = false

camip = app:option(Value, "camIP", translate("dj_camIP"))
camip.rmempty = false
user = app:option(Value, "usr", translate("dj_usr"))
user.rmempty = false
passwd = app:option(Value, "pwd", translate("dj_pwd"))
passwd.rmempty = false
passwd.password = true
cam_name = app:option(Value, "camName", translate("dj_camName"))
cam_name.rmempty = false

input_pref = app:option(Value, "inputPrefix", translate("dj_inputPrefix"))
input_pref.rmempty = true
m.uci:foreach("fstab", "mount",
		function(s)
			if s.target and #s.target > 0 then
				input_pref:value(s.target)
			end
		end)

is_save = app:option(Flag, "saveInput", translate("dj_saveInput"))

result_pref = app:option(Value, "resultPrefix", translate("dj_resultPrefix"))
result_pref.rmempty = true
m.uci:foreach("fstab", "mount",
		function(s)
			if s.target and #s.target > 0 then
				result_pref:value(s.target)
			end
		end)

result_fname = app:option(Value, "resultFilename", translate("dj_resultFilename"))
result_fname.rmempty = false

---- gpg ----
gpg = m:section(TypedSection, "gpg", translate("dj_section_gpg"))
gpg.anonymous = true

recipient = gpg:option(ListValue, "recipient", translate("dj_recipient"))
recipient:value("", translate("-- no key selected --"))
for _, k in ipairs(keys) do
	recipient:value(k.id, k.id .. " - " .. k.email)
end

extension = gpg:option(Value, "extension", translate("dj_extension"))
extension.rmempty = true

---- livecontrol ----
live = m:section(TypedSection, "livecontrol", translate("dj_section_livecontrol"))
live.anonymous = true

time = live:option(Value, "timeLimit", translate("dj_timeLimit"), translate("minutes", "Minutes"))
time.rmempty = false
polling = live:option(Value, "pollingWait", translate("dj_pollingWait"), translate("seconds", "Seconds"))
polling.rmempty = false

---- http ----
http = m:section(TypedSection, "http", translate("dj_section_http"))
http.anonymous = true

host = http:option(Value, "host", translate("dj_httphost"))
host.rmempty = false
is_secure = http:option(Flag, "isSecure", translate("dj_isSecure"))

---- debug ----
debug = m:section(TypedSection, "debug", translate("dj_section_debug"))
debug.anonymous = true

logfile = debug:option(Value, "logfile", translate("dj_logfile"))
loglevel = debug:option(Value, "loglevel", translate("dj_loglevel"))
loglevel:value("0", "0")
loglevel:value("1", "1")
loglevel:value("2", "2")
c_timeout = debug:option(Value, "connectTimeout", translate("dj_connectTimeout"), translate("seconds", "Seconds"))
r_timeout = debug:option(Value, "readTimeout", translate("dj_readTimeout"), translate("seconds", "Seconds"))

------------------------------- validater ------------------------------
function validate_string(self, value, _max)
	local max_val = tonumber(_max)

	if not max_val then
		max_val = 64
	end
	if value and #value > 0 and #value < max_val then
		return value
	else
		self.err_message = translate("error_string", "Error")
		return nil
	end
end

function validate_value(self, value, _min, _max)
	local min_val = _min
	local max_val = _max
	local v = tonumber(value)

	if not min_val then
		min_val = 0
	end
	if not max_val then
		max_val = 65535
	end
	if v and v >= min_val and v <= max_val then
		return value
	else
		self.err_message = translate("error_value", "Error") ..
				" (" .. translate("min") .. " " .. tostring(min_val) .. ", " ..
				translate("max") .. " " .. tostring(max_val) .. ")"
		return nil
	end
end

function validate_ip(self, value)
	if value and nw.isIPv4(value) then
		return value
	else
		self.err_message = translate("error_ip", "Error IP")
		return nil
	end
end

function validate_id(self, value)
	return validate_value(self, value, 1, 1000)
end

function validate_sleep(self, value)
	return validate_value(self, value, 1, 10)
end

function validate_max_preset(self, value)
	return validate_value(self, value, 1, 128)
end

function validate_preset_intvl(self, value)
	return validate_value(self, value, 1, 60)
end

function validate_ss_intvl(self, value)
	return validate_value(self, value, 2, 60)
end

function validate_time(self, value)
	return validate_value(self, value, 1, 60)
end

function validate_polling(self, value)
	return validate_value(self, value, 0, 60)
end

-- app
id.validate = validate_id
sleep.validate = validate_sleep
max_preset.validate = validate_max_preset
preset_intvl.validate = validate_preset_intvl
ss_intvl.validate = validate_ss_intvl

camip.validate = validate_ip
user.validate = validate_string
passwd.validate = validate_string
cam_name.validate = validate_string

input_pref.validate = validate_string
result_pref.validate = validate_string
result_fname.validate = validate_string

-- gpg
extension.validate = validate_string

-- livecontrol
time.validate = validate_time
polling.validate = validate_polling

-- http
host.validate = validate_ip
----------------------------------------------------------------------

return m
