--[[
LuCI support for MeshCruzer
Copyright (C) 2011 Thinktube Inc
]]--
local util = require "luci.util"
local uci  = require "luci.model.uci".cursor()
local sys  = require "luci.sys"
local web  = require "luci.tools.webadmin"
local nfs  = require "nixio.fs"
local ipairs, pairs, table, tonumber, string, io
		= ipairs, pairs, table, tonumber, string, io

module "luci.tools.network"

function single_ifs()
    local sin_ifs = {}
    uci:foreach("network", "interface",
		function(s)
			if s['.name'] ~= "loopback" then
				if s.type ~= "bridge" then
					sin_ifs[#sin_ifs+1] = s
				end
			end
		end)
    return sin_ifs
end

function wireless_ifs()
    local wif_ifs = {}
    uci:foreach("wireless", "wifi-iface",
		function(s)
			wif_ifs[#wif_ifs+1] = s
		end)
    return wif_ifs
end

function bridge_ifs()
    local br_ifs = {}
    uci:foreach("network", "interface",
		function(s)
			if s['.name'] ~= "loopback" then
				if s.type == "bridge" then
					br_ifs[#br_ifs+1] = s
				end
			end
		end)
    return br_ifs
end

function get_brinfo(s)
	local b = { }
	local m = false
	for l in util.execi("brctl show") do
		if not l:match("STP") then
			if m and l:match("^[a-z]") then
				break
			elseif m or l:match("^br%%-%s" % s['.name']) then
				m = true
				local r = util.split(l, "%s+", nil, true)
				if #r > 2 then
					b.name    = r[1]
					b.id      = r[2]
					b.stp     = r[3] == "yes"
					b.ifnames = { r[4] }
				else
					b.ifnames[#b.ifnames+1] = r[2]
				end
			end
		end
	end
	return b
end

-- name : "br-LAN"
function get_brinfo_by_brname(name)
	local b = { }
	local m = false
	for l in util.execi("brctl show") do
		if not l:match("STP") then
			if m and l:match("^[a-z]") then
				break
			elseif m or l:match("^br%%-%s" % name) then
				m = true
				local r = util.split(l, "%s+", nil, true)
				if #r > 2 then
					b.name    = r[1]
					b.id      = r[2]
					b.stp     = r[3] == "yes"
					b.ifnames = { r[4] }
				else
					b.ifnames[#b.ifnames+1] = r[2]
				end
			end
		end
	end
	return b
end

function get_ifname(s)
	return s.ifname and s.ifname:match("%S+")
end

function get_wlaninfo_by_ifname(ifname)
	local iw = {}

	if not ifname then return end

	-- search wifi-iface from device name like "wlan0"
	ret = uci:foreach("wireless", "wifi-iface",
		function(s)
			if s.ifname == ifname then
				iw.device = s.device
				iw.network = s.network or ""
				iw.encryption = s.encryption or "none"
				iw.mode = s.mode
				iw.ssid = s.ssid
				iw.ifname = s.ifname
			end
		end)

	-- get wifi-device using wifi-iface
	if ret then
		uci:foreach("wireless", "wifi-device",
			function(s)
				if s['.name'] == iw.device then
					iw.name = s['.name']
					iw.phy = s.phy
					iw.antenna = s.antenna
					iw.channel = s.channel
					iw.disabled = s.disabled or "0"
				end
			end)
	end

	return iw
end

function network_getinfo_by_network(network)
	local tbl = {}

	uci:foreach("network", "interface",
		function(s)
			if s['.name'] == network then
				tbl = s
			end
		end)
	return tbl
end

function get_ifaces4exsubnet()
	local tbl = {}
    local devinfo = sys.net.deviceinfo()

	-- bridge interface
	for _, b in ipairs(bridge_ifs()) do
		local br  = get_brinfo(b)
		local dev = br and br.name
		if br and devinfo and devinfo[dev] then
			local e = {}
			e.ifname  = dev  -- br-LAN
			e.netname = dev  -- br-LAN (it's same on this case)
			e.ipaddr = b.ipaddr or ""
			e.netmask = b.netmask or ""
			tbl[#tbl+1] = e
		end
	end

	-- normal interface
	for _, i in ipairs(single_ifs()) do
		local dev = get_ifname(i)
		if dev and devinfo and devinfo[dev] and not is_brport(dev) and
				not is_wifi(dev) and i.ipaddr and i.netmask and i['.name'] then
			local e = {}
			e.ifname = dev          -- ethX or wlanX
			e.netname = i['.name']  -- WAN, ADHOC_X, or AP_X
			e.ipaddr = i.ipaddr --or ""
			e.netmask = i.netmask --or ""
			tbl[#tbl+1] = e
		end
	end

	-- wifi interfaces
	for _, i in ipairs(wireless_ifs()) do
		local dev = get_ifname(i)
		local iw = get_wlaninfo_by_ifname(dev)
		if dev and devinfo and devinfo[dev] and not is_brport(dev) and is_wifi(dev) and
				iw and iw.mode == "ap" then
			local e = {}
			local ipinfo = network_getinfo_by_network(i.network)

			if ipinfo.ipaddr and ipinfo.netmask and i.network then
				e.ifname  = dev           -- ethX or wlanX
				e.radio   = i.device      -- RADIO_0, RADIO_1...
				e.netname = i.network     -- WAN, ADHOC_X, or AP_X
				e.ipaddr  = ipinfo.ipaddr --or ""
				e.netmask = ipinfo.netmask --or ""
				tbl[#tbl+1] = e
			end
		end
	end

	return tbl
end

function delete_alias(iface)
    local aliases = {iface}
    uci:foreach("network", "alias",
        function(section)
            if section.interface == iface then
                table.insert(aliases, section[".name"])
            end
        end)
    -- Delete Aliases and Routes
    uci:delete_all("network", "route", {interface=iface})
    uci:delete_all("network", "alias", {interface=iface})
end

function delete_alias_by_devname(iface_dev_name)
    return delete_alias(web.iface_get_network(iface_dev_name))
end

function isIPv4(address)
	if not address then return nil end
	local b1, b2, b3, b4 = string.match(address, "^(%d+)%.(%d+)%.(%d+)%.(%d+)$")

	b1 = tonumber(b1)
	b2 = tonumber(b2)
	b3 = tonumber(b3)
	b4 = tonumber(b4)

	if b1 and b1 <= 255 and b2 and b2 <= 255 and b3 and b3 <= 255 and b4 and b4 <= 255 then
		return address
	else
		return nil
	end
end

function isMAC(address)
	local function check_hex(value)
		if not value then return false end
		return string.match(value, "^[0-9a-fA-F][0-9a-fA-F]$")
	end

	local m1, m2, m3, m4, m5, m6 = string.match(address, "^(%x%x):(%x%x):(%x%x):(%x%x):(%x%x):(%x%x)$")
	if m1 and check_hex(m1) and m2 and check_hex(m2) and m3 and check_hex(m3) and
			m4 and check_hex(m4) and m5 and check_hex(m5) and m6 and check_hex(m6) then
		return true
	else
		return false
	end
end

function is_brport(entry)
    local devinfo = sys.net.deviceinfo()
    for _, b in pairs(bridge_ifs()) do
        local br = get_brinfo(b)
        local dev = br and br.name
        if dev and devinfo and devinfo[dev] then
            if string.find(table.concat(br.ifnames, " "), entry) then
                return true -- it's one of bridge interface
            end
        end
    end
    return false
end

function is_wifi(i)
	local wifi_ifs = {}

	uci:foreach("wireless", "wifi-iface",
		function(s)
			wifi_ifs[s.network or s.device] = true
		end)

	return wifi_ifs[i]
		or i:match("^wl%d+")
		or i:match("^ath%d+")
		or i:match("^wlan%d+")
end

function is_eth(e)
	return e:match("^eth%d+")
end